﻿Imports Microsoft.Xna.Framework
Imports Microsoft.Xna.Framework.Graphics

Namespace demo
    Public Class screenmanager
        Implements IScreenManager, IUpdateable, IDrawable, IGameComponent

        Public Game As Game
        Public SpriteBatch As SpriteBatch

        Private screens As List(Of Screen) 'list

        Private screensToDraw As Stack(Of Screen) 'stack

        Public Width As Integer
        Public Height As Integer

        Public Sub New(game_ As Game)
            Game = game_
            screens = New List(Of Screen)
            SpriteBatch = New SpriteBatch(game_.GraphicsDevice)
            screensToDraw = New Stack(Of Screen)
            Width = game_.GraphicsDevice.Viewport.Width
            Height = game_.GraphicsDevice.Viewport.Height

            'not sure the below is necessary as we never use services later on
            'game_.Services.AddService(GetType(IScreenManager), Me) 'gettype = typeof
            'will speed up the car ??? linked to ElapsedTime ?
            'game_.Components.Add(Me)
        End Sub

        Public Sub ShowScreen(Screen_ As Screen)

            If (screens.Contains(Screen_)) Then
                screens.Remove(Screen_)
            Else
                Screen_.ScreenManager = Me
                If (Screen_.State = ScreenState.Uninitialized) Then Screen_.Load(Nothing)
            End If
            Screen_.Show()
            screens.Add(Screen_)
        End Sub

        Public Sub HideScreen(screen_ As Screen)
            screen_.Hide()
        End Sub


        Public Sub CloseScreen(screen_ As Screen)
            screen_.Close()
        End Sub


        'region IUpdateable Members

        Public Sub Draw(gameTime As GameTime) Implements IDrawable.Draw
            SpriteBatch.Begin()
            While (screensToDraw.Count > 0)

                Dim screen_ As Screen = screensToDraw.Pop()
                screen_.Draw(gameTime)
            End While
            SpriteBatch.End()
        End Sub

        Public ReadOnly Property DrawOrder As Integer Implements IDrawable.DrawOrder
            Get
                Return 0
            End Get
        End Property

        Public Event DrawOrderChanged(sender As Object, e As EventArgs) Implements IDrawable.DrawOrderChanged

        Public ReadOnly Property Visible As Boolean Implements IDrawable.Visible
            Get
                Return True
            End Get
        End Property

        Public Event VisibleChanged(sender As Object, e As EventArgs) Implements IDrawable.VisibleChanged

        Public Sub Initialize() Implements IGameComponent.Initialize

        End Sub

        Public ReadOnly Property Enabled As Boolean Implements IUpdateable.Enabled
            Get
                Return True
            End Get
        End Property

        Public Event EnabledChanged(sender As Object, e As EventArgs) Implements IUpdateable.EnabledChanged

        Public Sub Update(gameTime As GameTime) Implements IUpdateable.Update
            Dim current As Screen
            screensToDraw.Clear()
            Dim i As Integer = screens.Count - 1
            For i = screens.Count - 1 To 0 Step -1
                current = screens(i)
                current.Update(gameTime)
                If (current.State <> ScreenState.Inactive And current.State <> ScreenState.Closed) Then

                    screensToDraw.Push(current)
                    If (current.IsPopup = False) Then Exit For
                End If

                If (current.State = ScreenState.Closed) Then screens.RemoveAt(i)

            Next
        End Sub

        Public ReadOnly Property UpdateOrder As Integer Implements IUpdateable.UpdateOrder
            Get
                Return 1
            End Get
        End Property

        Public Event UpdateOrderChanged(sender As Object, e As EventArgs) Implements IUpdateable.UpdateOrderChanged
    End Class
End Namespace